/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2023 CEA/DEN
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   TLauncherEnv.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2023-02-22
 
 \brief  working dir for launcher code
*/

#ifndef __LAUNCHERENV__
#define __LAUNCHERENV__

// ROOT
#include "Rtypes.h"
#include "TString.h"
#include "TNamed.h"

// URANIE
#include "Launcher.h"
#include "TCode.h"
#include "UExceptions.h"


#include <iostream>
#include <fstream>
#include <iomanip>

using std::cout;
using std::endl;
using std::cerr;
using std::setw;

namespace URANIE {
namespace Launcher {

class TLauncherEnv: public TNamed
{
protected:
    TCode* _code;
    TString _sdirectory;        ///< Current directory
    TString _sworkingDirectory; ///< The Working directory
    Bool_t _blog;               ///< Boolean to edit the log
    
    
    //! choose the working directory
    void verifyWorkingDirectory();
    //! Builts Results directory, no available options
    void createResultsDirectory();
    //! clean the working directory
    bool cleanWorkingDirectory();
    
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    TLauncherEnv();
    TLauncherEnv(TCode *code);
    virtual ~TLauncherEnv();
    
    //---------------------------------------------
    /** @name use working directory
     **/
    //! set the working directory
    void setWorkingDirectory(TString str);
    //! Gets the working directory
    /*!
     \return (TString) the working directory
     */
    TString getWorkingDirectory()
    {
        return _sworkingDirectory;
    }

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    void setLog()
    {
        _blog = kTRUE;
    }
    void unsetLog()
    {
        _blog = kFALSE;
    }
    void changeLog()
    {
        _blog = _blog ? kFALSE : kTRUE;
    }
    Bool_t getLog()
    {
        return _blog;
    }
    //@}
    
    ClassDef(URANIE::Launcher::TLauncherEnv, ID_LAUNCHER)

};

}} // namespace

#endif // __LAUNCHERENV__
