//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2025 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TFCNCode.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   mar jui 3 10:38:56 CEST 2007
 
 \brief Example of a function to be minimized which calls an external code TCode and a dataserver 


 This is tested for Rosenbrock function defined as follows:
 \em Rosenbrock function is d�fined by the equation
 \f[
 f(x, y) \, = \, 100( y - x^{2})^{2} + ( 1 - x )^{2}
 \f]

 The minimun is reached when \f$ ( \star{x^{0}}, \star{y^{0}} ) \, = \, ( 1.0, 1.0 ) \f$

 A possible beginning point may be : \f$ ( x^{0}, y^{0} ) \, = \, ( -1.2, 1.0 )\f$ with f equal to 24.20.
 */
#ifndef TFCNCODE_H
#define TFCNCODE_H

#include "TH1.h"
#include "TF1.h"
#include "TFile.h"
#include "TPad.h"
#include "TRandom3.h"
#include "TVirtualFitter.h"
#include "TStyle.h"
#include "Minuit2/FCNBase.h"

#include <vector>
#include <iostream>

// URANIE
#include "Optimizer.h"
#include "TCode.h"
#include "TDataServer.h"

namespace URANIE
{
namespace Optimizer
{
class TFCNCode: public ROOT::Minuit2::FCNBase
{

public:
    TFCNCode() :
            _ninput(0), _noutput(0), _soutput(""), _tds(NULL), _code(NULL)
    {
    }
    //! Constructor with a dataserver
    TFCNCode(URANIE::DataServer::TDataServer *tds,
            URANIE::Launcher::TCode *code, TString scost) :
             _sCost(scost), _tds(tds), _code(code)
    {
        init();
    }

    ~TFCNCode()
    {
        _dinputValue = NULL;
        delete[] _dinputValue;
        _doutputValue = NULL;
        delete[] _doutputValue;

    }

    void init();

    double operator()(const vector<double> & par) const;

    double Up() const
    {
        return 1.;
    }

    void clean();

private:
    int _ninput;
    double * _dinputValue;
    int _noutput;
    TString _soutput;                         ///< Name of the output attributes
    double * _doutputValue;                    ///< The output attribute name
    TString _sCost; ///< The name of the selected cost (the number must be equal to 1)
    Int_t _nCost; ///< the current position for the selected cost in the output list of attribute
    URANIE::DataServer::TDataServer *_tds;     //! Pointeur vers un TDS
    URANIE::Launcher::TCode *_code;    //! Pointeur vers un TCode

    ClassDef(URANIE::Optimizer::TFCNCode, ID_OPTIMIZER)
    //Classe de

};
} // Fin du namespace Optimizer
} // Fin du namespace URANIE
#endif
