//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2025 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TSumOfSquares.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   tue jul 3 10:38:56 CEST 2007
 
 \brief Example of a function to be minimised : Sum of Square
 
 \par Modifications
 \li Fabrice Gaudier (2010-04-12 14:30:47)
 Add Log
 */
#ifndef TSUMOFSQUARES_H
#define TSUMOFSQUARES_H

// ROOT
#include "TH1.h"
#include "TF1.h"
#include "TPad.h"
#include "TRandom3.h"
#include "TVirtualFitter.h"
#include "TStyle.h"
#include "Minuit2/FCNBase.h"
#include "TMatrixT.h"

#include <vector>
#include <iostream>

// URANIE
#include "Optimizer.h"
#include "TCode.h"
#include "TDataServer.h"
#include "TObjective.h"

namespace URANIE
{
namespace Optimizer
{
class TSumOfSquares: public ROOT::Minuit2::FCNBase
{
private:
    vector<TObjective *> _objectives;              ///< vector of objectives

public:

    TSumOfSquares() :
             _objectives(0), _ninput(0), _noutput(0), _tds(NULL), _code(NULL), _sCriteria("")
    {
    }
    //! Constructor with a dataserver
    TSumOfSquares(URANIE::DataServer::TDataServer *tds,
            URANIE::Launcher::TCode *code) :
            _objectives(0), _tds(tds), _code(code), _sCriteria("") 
    {
        init();
    }

    ~TSumOfSquares()
    {
        _dinputValue = NULL;
        delete[] _dinputValue;
        _doutputValue = NULL;
        delete[] _doutputValue;

    }

    void init();

    double operator()(const vector<double> & par) const;
    
    double Up() const
    {
        return 1.;
    }

    void clean();

    void addObjective(TObjective *tobj)
    {
        _objectives.push_back(tobj);
    }
    Int_t getNObjectives()
    {
        return (Int_t)(_objectives.size());
    }

private:
    int _ninput;
    double * _dinputValue;
    int _noutput;
    double * _doutputValue;
    URANIE::DataServer::TDataServer *_tds;               //! Pointer vers un TDS
    URANIE::Launcher::TCode *_code;                //! Pointer vers un TCode
    TList *_listOfDataOutput;    //! List of matrix
    TString _sCriteria;      //! The name of the attribute which is the criteria

    ClassDef(URANIE::Optimizer::TSumOfSquares, ID_OPTIMIZER)
    //Classe de
};
} // Fin du namespace Optimizer
} // Fin du namespace URANIE
#endif

