////////////////////////////////////////////////////////////// 
// Copyright (C) 2013-2025 CEA/DES 
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
\file   TCramerVonMises.h
\author Rudy Chocat <rudy.chocat@cea.fr>
\date   2024-09-09 11:19:38

\brief Interface of class URANIE::Sensitivity::TCramerVonMises

*/
/**

 * @class URANIE::Sensitivity::TCramerVonMises
 *
 * @brief Description of the class TCramerVonMises.
 *
 * To be written by the developper.
 *
 */
#ifndef TCramerVonMises_H
#define TCramerVonMises_H
#include "TString.h"
#ifndef ROOT_TMethodCall
#include "TMethodCall.h"
#endif
#ifndef ROOT_TNtupleD
#include "TNtupleD.h"
#endif

// Uranie
#include "TStochAttribut.h"
#include "TSensitivity.h"
#include "TSobol.h"
#include "TSampling.h"
#include "TQMC.h"

using std::cout;
using std::endl;

namespace URANIE {
namespace Sensitivity {
class TCramerVonMises: public TSobol {


public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TCramerVonMises();
    //! Default constructor with the TDataServer
    /*
     \param tds (URANIE::DataServer::TDataServer *) the dataserver
     \param fcn (void *) The function to analyze
     \param ns (Int_t) The size (total) of the sampling
     \param varexpinput(const char*)[""] The list of input attributes to pass to the function separated by the caracter ":"
     \param varexpoutput (const char *)[""]  The list of output attributes separated by the caracter ":"
     \param
     */
    TCramerVonMises(URANIE::DataServer::TDataServer *tds, void (*fcn)(Double_t*,Double_t*),
            const char *varexpinput, const char *varexpoutput,
            Int_t ns, Option_t * option = "");
    //! Default constructor with the TDataServer
    /*
     \param tds (URANIE::DataServer::TDataServer *) the dataserver
     \param fcn (const char*) The function to analyze
     \param ns (Int_t) The size (total) of the sampling
     \param varexpinput(const char*)[""] The list of input attributes to pass to the function separated by the caracter ":"
     \param varexpoutput (const char *)[""]  The list of output attributes separated by the caracter ":"
     \param
     */
    TCramerVonMises(URANIE::DataServer::TDataServer *tds, const char *fcn,
     Int_t ns, const char *varexpinput, const char *varexpoutput,
     Option_t* option="");
    //! Default constructor with a TDataServer filling
    TCramerVonMises(URANIE::DataServer::TDataServer *tds, const char *varexpinput,
            const char *varexpoutput, Option_t * option = "");

    //! Default constructor with a tcode
    TCramerVonMises(URANIE::DataServer::TDataServer *tds, URANIE::Launcher::TCode *fcode,
            Int_t ns, Option_t * option = "");
    //! Default constructor with a trun
    TCramerVonMises(URANIE::DataServer::TDataServer *tds, URANIE::Relauncher::TRun *run,
            Int_t ns, Option_t * option = "");
    //! Default destructor
    virtual ~TCramerVonMises();

    ClassDef(URANIE::Sensitivity::TCramerVonMises, ID_SENSITIVITY);

    

        virtual void generateSample(Option_t * option = "");

        void evaluateIndexes(Option_t * option = "");

    void drawIndexes(TString sTitre, const char *select = "", Option_t * option = "");


  TMatrixD _matO;             ///< The matrix M
TMatrixD _valO;


};

} // End Of namespace Sensitivity
} // End Of namespace URANIE
#endif

