////////////////////////////////////////////////////////////// 
// Copyright (C) 2013-2025 CEA/DES 
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TCRAMERVONMISES_H
#define CPPUNITEST_TCRAMERVONMISES_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TSensitivity.h"
#include "TCramerVonMises.h"
#include "TDataServer.h"
#include "TNormalDistribution.h"
#include "UExceptions.h"

#include <memory>

class TCramerVonMisesTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TCramerVonMisesTest);

    CPPUNIT_TEST (testCramerVonMisesTest);
    CPPUNIT_TEST (testCramerVonMisesFunctionRelauncher);
    CPPUNIT_TEST (testIshigamiCramerVonMisesWithDrawings);
    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TCramerVonMisesTest();

    //! Default destructor
    virtual ~TCramerVonMisesTest();

    void setUp();
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{ 
    // void testCramerVonMises();

    void testCramerVonMisesTest();
    void testCramerVonMisesFunctionRelauncher();
    void testIshigamiCramerVonMisesWithDrawings();
    // void throwExceptionConstructorTooLongOutputFunctionFirst();
    // void throwExceptionConstructorTooLongOutputFunctionSecond();
    // void throwExceptionConstructorTooLongOutputCode();
    // void throwExceptionConstructorTooLongOutputRun();
    // void throwExceptionConstructorTooLongOutputFromData();
    // void throwExceptionTRegressionConstructorTdsNull();
    // void throwExceptionTRegressionConstructorFalseInput();
    // void throwExceptionTRegressionConstructorFalseOutput();
    //@}

private:
    std::unique_ptr<URANIE::DataServer::TNormalDistribution> _x1, _x2;
    URANIE::DataServer::TDataServer *_tds1, *_tds;
    URANIE::Sensitivity::TCramerVonMises *_tcvm;
    URANIE::Sensitivity::TCramerVonMises *_tcvm1;
};

#endif  // CPPUNITEST_TCRAMERVONMISES_H