/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2023 CEA/DEN
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   TCodeAsEval.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2023-03-16
 
 \brief  using launcher TCode as a TEval
*/

#ifndef __TCODEASEVAL__
#define __TCODEASEVAL__

//ROOT
#include "Rtypes.h"
#include "TString.h"
#include "TList.h"

//URANIE
#include "Relauncher.h"
#include "TShareEval.h"
#include "TLauncherEnv.h"
#include "TCode.h"

namespace URANIE
{
namespace Relauncher
{

class TCodeAsEval : public TShareEval, public URANIE::Launcher::TLauncherEnv
{
protected:
    URANIE::Launcher::forkcode *_fjob;
    vector<URANIE::DataServer::UEntry*> _inputValues, _outputValues;

public:
    TCodeAsEval(URANIE::Launcher::TCode *);
    virtual ~TCodeAsEval();
    
    void setJob(URANIE::Launcher::forkcode *);
    void prepareEntries(URANIE::Launcher::forkcode *, vector<URANIE::DataServer::UEntry*> *);
        
    // protocole
    virtual void freeze();
    virtual TEval *rebuild(int rank, Bool_t chef = kTRUE);
    virtual Bool_t unbuild();
    virtual int eval(vector<URANIE::DataServer::UEntry*>*, vector<URANIE::DataServer::UEntry*>*, int ncalc=0);
    
    ClassDef(URANIE::Relauncher::TCodeAsEval, ID_RELAUNCHER)
};

}} // namespace

#endif
