/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2024 CEA/DEN
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   TCodeBase.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2024-07-02
 
 \brief  gestion des répertoires fichiers de travail*
*/

#ifndef __TCodeBase__
#define __TCodeBase__

//ROOT
#include "Rtypes.h"
#include "TObjArray.h"

// Uranie
#include "Relauncher.h"
#include "TShareEval.h"
#include "TDataServer.h"


namespace URANIE {
namespace Relauncher {


/* for composition */
/* manage workdir */
class TDirEval
{
protected:
    Bool_t _isnumered;
    Bool_t _keepAllFolders;
    TString _initialdir, _workdir, _tmpdir;

    void buildWorkDir(void);
    void buildTmpDir(int id, Bool_t ch=kTRUE);
    
public:
    TDirEval();
    virtual ~TDirEval();
    
    void setWorkingDir(TString dir);
    void setOldTmpDir();
    void keepAllFolders();

    //ClassDef(URANIE::Relauncher::TDirEval, ID_RELAUNCHER)
};

/* manage input and aouput file */
class TCodeScript: public TObject
{
public:
    virtual void build(TString &dir, TObjArray *in) = 0;
    virtual void put(TString &ref, TString &tmp, Double_t *in) = 0;
    virtual void put(TString &ref, TString &tmp, vector<URANIE::DataServer::UEntry*> *in) = 0;

//    std::stringstream buffer;
    
    ClassDef(URANIE::Relauncher::TCodeScript, ID_RELAUNCHER)
};

class TCodeResult: public TObject
{
public:
    virtual void build(TObjArray *) = 0;
    virtual int get(TString &, Double_t*) = 0;
    //    virtual int get(TString &, Double_t**) = 0;
    virtual int get(TString &work, vector<URANIE::DataServer::UEntry*> *sout) = 0;

    virtual void clearFile(TString tmpdir) = 0;
    
//    std::stringstream buffer;
    
    ClassDef(URANIE::Relauncher::TCodeResult, ID_RELAUNCHER)
};

class TIOFileEval : public TDirEval
{
protected:
    Bool_t  _clone;
    TString _localdir;
    TList *_scripts;
    TCodeResult *_result;

    void buildDir(void);

public:
    TIOFileEval();
    virtual ~TIOFileEval();

    void addInputFile(TCodeScript *fil);
    void addOutputFile(TCodeResult *fil);
    void setReference(TString dir);
    void setClone();

    virtual void setTmpDir(int id, Bool_t chef);
    virtual int callFunction() = 0;
    
    //ClassDef(URANIE::Relauncher::TIOFIleEval, ID_RELAUNCHER)
};

class TCodeBase: public TShareEval, public TIOFileEval
{
protected:
    int putAndRun(Double_t *, int ncalc=-1);
    int putAndRun(vector<URANIE::DataServer::UEntry*>*, int ncalc=-1);

public:
    TCodeBase();
    virtual ~TCodeBase(void);

    virtual void freeze(void);
    virtual int eval(vector<URANIE::DataServer::UEntry*>*, vector<URANIE::DataServer::UEntry*>*, int =0 );

    ClassDef(URANIE::Relauncher::TCodeBase, ID_RELAUNCHER)
};


}} // namespace

#endif
