//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2025 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.7 $
// $State$
//////////////////////////////////////////////////////////////
/*!
  \file   TCodeEval.cxx
  \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
  \date    2011-04-04.15.35.52

  \brief   encapsulation d'un TCode
*/

#include "TCodeEval.h"

#ifndef WIN32
#include <sys/wait.h>
#else
#include <io.h>
#endif

//ROOT
#include "TSystem.h"
//#include "TString.h"

//URANIE
#include "UExceptions.h"

ClassImp (URANIE::Relauncher::TCodeEval);

namespace URANIE
{
namespace Relauncher
{

// TCodeEval

/** constructeur destructeur **/

TCodeEval::TCodeEval(const char *scmd) :
    TCodeBase(),
#ifdef WIN32
    _cmd(TString::Format("cd %%D && %s", scmd))
#else
    _cmd(TString::Format("cd %%D ; %s", scmd))
#endif	
{
    _origcmd=_cmd;
}


TCodeEval::~TCodeEval()
{
}

/** configuration **/
void TCodeEval::setTmpDir(int id, Bool_t chef)
{
    /* dir */
    TIOFileEval::setTmpDir(id, chef);

    /* commande */
    if( _keepAllFolders )
        _cmd = _origcmd;
    _cmd.ReplaceAll("%D", _tmpdir);
}


/** protocole **/
TEval * TCodeEval::rebuild(int id, Bool_t chef)
{
    if (chef)
    {
        TCodeEval *ret = new TCodeEval(*this);
        ret->setClone();
        if( ret->_keepAllFolders == false)
            ret->setTmpDir(id, chef);
        return (TEval *) ret;
    }
    else {
        _subeval = true ;
        return (TEval *) this;
    }
}

Bool_t TCodeEval::unbuild()
{
    return kTRUE;
    //return (!_tmpdir.IsNull())? kTRUE : kFALSE ;
    //if (!_tmpdir.EqualTo("")) { printf("TCodeEval unbuild\n"); delete this;}
}

int TCodeEval::callFunction()
{
    int ret;
    
    ret = system(_cmd.Data());

#ifndef WIN32	
    if (WEXITSTATUS(ret))
        cout  << "Command " << _cmd.Data() << " has returned non-zero exit code ("<<WEXITSTATUS(ret)<<").\n  If any different from 127 (usually for unknown command) and 139 (usually for SIGSEV), the exit code meaning is \"command\" dependent."<<endl;
    // cout<<"ret is "<<ret<<" => "<<"WIFEXITED(ret)::"<<WIFEXITED(ret)<<"; "<<"WEXITSTATUS(ret)::"<<WEXITSTATUS(ret)<<"; "
    //     <<"WIFSIGNALED(ret)::"<<WIFSIGNALED(ret)<<"; "<<"WTERMSIG(ret)::"<<WTERMSIG(ret)<<"; "<<"WCOREDUMP(ret)::"<<WCOREDUMP(ret)<<"; "
    //     <<"WIFSTOPPED(ret)::"<<WIFSTOPPED(ret)<<"; "<<"WSTOPSIG(ret)::"<<WSTOPSIG(ret)<<"; "<<endl;
#endif
    return ret;
}


}} //namespace

