//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2025 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.7 $
// $State$
//////////////////////////////////////////////////////////////
/*!
  \file   TCodeEval.cxx
  \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
  \date    2011-04-04.15.35.52

  \brief   encapsulation d'un TCode
*/

#include "TLibCodeEval.h"

#ifndef WIN32
#include <sys/wait.h>
#else
#include <io.h>
#endif

//ROOT
#include "TSystem.h"

//URANIE
#include "UExceptions.h"

ClassImp (URANIE::Relauncher::TLibCodeEval);
ClassImp (URANIE::Relauncher::TCJitCodeEval);

namespace URANIE
{
namespace Relauncher
{

/** TLibCodeEval **/
TLibCodeEval::TLibCodeEval() : TCodeBase() { }

TLibCodeEval::~TLibCodeEval() { }

TEval * TLibCodeEval::rebuild(int id, Bool_t chef)
{
    TLibCodeEval *ret = clone();
    ret->setClone();
    if (ret->_keepAllFolders == false) ret->setTmpDir(id, chef);
    if (!chef) ret->_subeval = true;
    return (TEval *) ret;
}

Bool_t TLibCodeEval::unbuild()
{
    return kTRUE;
}

int TLibCodeEval::callFunction()
{
    int ret=1;
    
    /* call */
    gSystem->cd(_tmpdir);
    try {
        ret = runCode();
    }
    catch(...) {
        ret = 0;
    }
    gSystem->cd(_initialdir);
    return 0;

}

/** TCJitCodeEval **/
TCJitCodeEval::TCJitCodeEval(void (*fun)()) : TCodeBase(),_fun(fun) { }

TCJitCodeEval::~TCJitCodeEval() { }

TEval * TCJitCodeEval::rebuild(int id, Bool_t chef)
{
    TCJitCodeEval *ret = new TCJitCodeEval(*this);
    ret->setClone();
    if (ret->_keepAllFolders == false) ret->setTmpDir(id, chef);
    if (!chef) ret->_subeval = true;
    return (TEval *) ret;
}

Bool_t TCJitCodeEval::unbuild()
{
    return kTRUE;
}

int TCJitCodeEval::callFunction()
{
    int ret = 1;
    
    /* call */
    gSystem->cd(_tmpdir);
    try {
        (*_fun)();
    }
    catch(...) {
        ret = 0;
    }
    gSystem->cd(_initialdir);
    return ret;

}

}} //namespace

