//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2025 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.7 $
// $State$
//////////////////////////////////////////////////////////////
/*!
  \file   TCodeEval.cxx
  \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
  \date    2011-04-04.15.35.52

  \brief   encapsulation d'un TCode
*/

#include "TPyCodeEval.h"

#ifndef WIN32
#include <sys/wait.h>
#else
#include <io.h>
#endif

//ROOT
#include "TSystem.h"

//URANIE
#include "UExceptions.h"

ClassImp (URANIE::Relauncher::TPyCodeEval);

namespace URANIE
{
namespace Relauncher
{

TPyCodeEval::TPyCodeEval(PyObject *fun) : TCodeBase(),_fun(fun) { }

TPyCodeEval::~TPyCodeEval() { }

TEval * TPyCodeEval::rebuild(int id, Bool_t chef)
{
    TPyCodeEval *ret = new TPyCodeEval(*this);
    ret->setClone();
    if (ret->_keepAllFolders == false) ret->setTmpDir(id, chef);
    if (!chef) ret->_subeval = true;
    return (TEval *) ret;
}

Bool_t TPyCodeEval::unbuild()
{
    return kTRUE;
}

int TPyCodeEval::callFunction()
{
    int ret;
    PyObject *pArgs, *pRetour;
    //PyGILState_STATE gstate;
    
    /* empty arg */
    pArgs = PyTuple_New(0);
    
    /* acquire GIL */
    // unneeded thread is forbiden
    //gstate = PyGILState_Ensure();
    
    /* call */
    gSystem->cd(_tmpdir);
    pRetour = PyObject_CallObject(_fun, pArgs);
    Py_DECREF(pArgs);
    gSystem->cd(_initialdir);

    /* works ? */
    if (pRetour == NULL) {
        fprintf(stderr, "Call failed\n");
        //PyGILState_Release(gstate);
        return 1;
    }
    
    Py_DECREF(pRetour);
    /* release GIL */
    //PyGILState_Release(gstate);
    return 0;

}

}} //namespace

