// --- C++ ---
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2024 CEA/DEN
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   IshigamiLib.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2024-07-11
 
 \brief  ishigami exemple as a TLibCodeEval
*/

#include "IshigamiLib.h"

#include "math.h"

#define SIZEIN 3
#define FPREC 10
// ishigami coef
#define ACOEF 7.0
#define BCOEF 0.1

// IshigamiLib class
IshigamiLib::IshigamiLib() : URANIE::Relauncher::TLibCodeEval() {}

IshigamiLib::~IshigamiLib() {}

URANIE::Relauncher::TLibCodeEval *IshigamiLib::clone()
{
    return new IshigamiLib(*this);
}


int IshigamiLib::runCode()
{
    double vin[SIZEIN], ret;
    TString line;
    int i;
    
    // read input parameters
    std::ifstream fin("ishigamin.dat", std::ios::in);
    for (i=0; line.ReadLine(fin) && i<SIZEIN; i++) {
        vin[i] = line.Atof();
    }
    fin.close();
    // Ishigami eval
    ret = sin(vin[0]);
    ret += ACOEF * sin(vin[1]) * sin(vin[1]);
    ret += BCOEF * vin[2] * vin[2] * vin[2] * vin[2] * sin(vin[0]);
    // write output result
    std::ofstream fout("ishigamout.dat", std::ios::out);
    fout.precision(FPREC);
    fout << ret << std::endl;
    fout.close();
    //
    return 1;
}

