#!/usr/bin/env python

import ROOT
import math

def ishigami(x1, x2, x3) :
    """ Ishigami function """
    # const
    a = 7.0
    b = 0.1
    # calculate
    res = math.sin(x1)
    res += a * math.sin(x2) * math.sin(x2)
    res += b * x3 * x3 * x3 * x3 * math.sin(x1)
    return [res,]

def ishigamiCode() :
    # read input file
    pin = []
    with open("ishigin.dat", "r") as fin :
        pin = fin.readlines()
    x1 = float(pin[0].strip())
    x2 = float(pin[1].strip())
    x3 = float(pin[2].strip())
    # calculate
    res = ishigami(x1, x2, x3)
    # write ouput file
    with open("ishigout.dat", "w") as fout :
        fout.write("%10.8g\n"%res[0])



# ------------------------------------------------------------------------------

# output attribute
ishi = ROOT.URANIE.DataServer.TAttribute("Ishigami")

# tds
tds = ROOT.URANIE.DataServer.TDataServer("tdsishigami", "tds for Python Ishigami")
tds.fileDataRead("Ishigami.dat")

# get input attributes from tds
x1 = tds.getAttribute("x_{1}")
x2 = tds.getAttribute("x_{2}")
x3 = tds.getAttribute("x_{3}")

# input file
fin = ROOT.URANIE.Relauncher.TLineScript("ishigin.dat")
fin.addInput(x1)
fin.addInput(x2)
fin.addInput(x3)
# output file
fout = ROOT.URANIE.Relauncher.TLineResult("ishigout.dat")
fout.addOutput(ishi)
# code
code = ROOT.URANIE.Relauncher.TPyCodeEval(ishigamiCode)
code.addInputFile(fin)
code.addOutputFile(fout)

# run
run = ROOT.URANIE.Relauncher.TSequentialRun(code)
run.startSlave()
if (run.onMaster()) :
    # launcher
    launch = ROOT.URANIE.Relauncher.TLauncher2(tds, run)
    launch.solverLoop()

    tds.exportData("_output_testIshigamiPycodeSeq_.dat");

    run.stopSlave()

